#include <gtk/gtk.h>
#include <glade/glade-xml.h>
#include <gconf/gconf-client.h>
#include "tinylemming.h"
#include <config.h>

GladeXML  *glade;
GtkWidget *window;
gboolean save_prefs;

void save_toggled_cb (GtkButton *button, gpointer data) {
	save_prefs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
}

TinyLemming * lemming_prefs_dialog ( void ) {

	TinyLemming 	*lemming = lemming_new();
	GtkWidget	*apply;
	GtkWidget	*cancel;
	GtkWidget	*save;
	GConfClient	*client = gconf_client_get_default ();

	glade = glade_xml_new (PACKAGE_DATA_DIR 
			       "/tinylemming/lemming-prefs.glade",
			       "prefs_window", NULL);
	window = glade_xml_get_widget (glade,"prefs_window");
	apply = glade_xml_get_widget (glade,"apply");
	cancel = glade_xml_get_widget (glade,"cancel");
	save = glade_xml_get_widget (glade,"save-prefs");

	g_object_set_data (G_OBJECT(window),"prefs",lemming);

	g_signal_connect (G_OBJECT(window),"destroy-event",
			  G_CALLBACK(lemming_prefs_cancel),NULL);
	g_signal_connect (G_OBJECT(window),"delete-event",
                          G_CALLBACK(lemming_prefs_cancel),NULL);
	g_signal_connect (G_OBJECT(apply),"clicked",
                          G_CALLBACK(lemming_prefs_apply),NULL);
	g_signal_connect (G_OBJECT(cancel),"clicked",
                          G_CALLBACK(lemming_prefs_cancel),NULL);
	g_signal_connect (G_OBJECT(save),"toggled",
			  G_CALLBACK(save_toggled_cb),NULL);
	lemming_prefs_load (client);
	gtk_widget_show_all (window);
	gtk_main ();
	lemming = g_object_get_data (G_OBJECT(window),"prefs");
	lemming = lemming_prefs_validate (lemming);	
	if (save_prefs) lemming_prefs_save (client,lemming);

	gtk_widget_destroy(window);
	return lemming;
}

void lemming_prefs_load (GConfClient *client) {

	GtkWidget *nick_box = glade_xml_get_widget (glade, "nick_box");
	GtkWidget *real_box = glade_xml_get_widget (glade, "real_box");
	GtkWidget *server_box = glade_xml_get_widget (glade, "server_box");
	GtkWidget *port_box = glade_xml_get_widget (glade, "port_box");
	GtkWidget *channel_box = glade_xml_get_widget (glade, "channel_box");
	gchar *nick;
	gchar *real;
	gchar *server;
	gchar *channel;
	gint port;

	nick = gconf_client_get_string (client, LEMMING_NICK_KEY, NULL);
	if (nick == NULL) nick = (gchar *)getenv("USER");
	gtk_entry_set_text (GTK_ENTRY(nick_box),nick);
	real = gconf_client_get_string (client, LEMMING_REAL_KEY, NULL);
	if (real == NULL) real = g_strdup(nick);
	gtk_entry_set_text (GTK_ENTRY(real_box),real);
	server = gconf_client_get_string (client, LEMMING_SERVER_KEY, NULL);
	if (server == NULL) server = g_strdup("irc.freenode.net");
	gtk_entry_set_text (GTK_ENTRY(server_box),server);
	port = gconf_client_get_int (client, LEMMING_PORT_KEY, NULL);
	if (port < 1 || port > 65535) port = 6667;
	gtk_entry_set_text (GTK_ENTRY(port_box), g_strdup_printf("%d",port));
	channel = gconf_client_get_string (client, LEMMING_CHANNEL_KEY, NULL);
	if (channel == NULL) channel = g_strdup ("#lemmings");
	gtk_entry_set_text (GTK_ENTRY(channel_box),channel);
	return;
}

void lemming_prefs_apply (GtkWidget *widget, gpointer data) {
	GtkWidget *nick_box = glade_xml_get_widget (glade, "nick_box");
        GtkWidget *real_box = glade_xml_get_widget (glade, "real_box");
        GtkWidget *server_box = glade_xml_get_widget (glade, "server_box");
        GtkWidget *port_box = glade_xml_get_widget (glade, "port_box");
        GtkWidget *channel_box = glade_xml_get_widget (glade, "channel_box");
	TinyLemming *lemming = g_object_get_data (G_OBJECT(window),"prefs");
	lemming->nick = g_strdup(gtk_entry_get_text (GTK_ENTRY(nick_box)));
	lemming->real_name = g_strdup (gtk_entry_get_text 
						(GTK_ENTRY(real_box)));
	lemming->server = g_strdup(gtk_entry_get_text (GTK_ENTRY(server_box)));
	lemming->port = atoi(gtk_entry_get_text (GTK_ENTRY(port_box)));
	lemming->channel = g_strdup(gtk_entry_get_text 
						(GTK_ENTRY(channel_box)));
	gtk_main_quit();
	g_object_set_data (G_OBJECT(window),"prefs",lemming);
	return;
}

void lemming_prefs_cancel (GtkWidget *widget, gpointer data) {
	gtk_main_quit();
	exit(0);
}

void lemming_prefs_save (GConfClient *client, TinyLemming *lemming) {
	gconf_client_set_string (client, LEMMING_NICK_KEY, lemming->nick, NULL);
	gconf_client_set_string (client, LEMMING_REAL_KEY, 
						lemming->real_name, NULL);
	gconf_client_set_string (client, LEMMING_SERVER_KEY, 
				 lemming->server, NULL);
	gconf_client_set_int (client, LEMMING_PORT_KEY, lemming->port, NULL);
	gconf_client_set_string (client, LEMMING_CHANNEL_KEY, lemming->channel, NULL);
}

TinyLemming * lemming_prefs_validate (TinyLemming *lemming) {

	int port = lemming->port;
	if (lemming->nick == NULL || lemming->nick[0]=='\0') {
		lemming->nick = g_strdup ((gchar *)getenv("USER"));
	}

	if (lemming->real_name == NULL || lemming->real_name[0]=='\0') {
		lemming->real_name = g_strdup ((gchar *)getenv("USER"));
	}

	if (lemming->server == NULL || lemming->server[0]=='\0') {
		lemming->server = g_strdup ("irc.freenode.net");
	}

	if (port < 0 || port > 65535) lemming->port = 6667;

	if (lemming->channel == NULL || lemming->channel[0]=='\0') {
		lemming->channel == g_strdup ("#lemmings");
	}

	if (! g_str_has_prefix (lemming->channel,"#")) {
		gchar *newchan = g_strdup_printf("#%s",lemming->channel);
		lemming->channel = g_strdup (newchan);
		g_free (newchan);
	}

	return lemming;
}
