#include <gtk/gtk.h>
#include <glib.h>
#include <libgtcpsocket/gtcp-socket.h>
#include <libgircclient/girc-client.h>
#include <libgircclient/girc-user.h>
#include "tinylemming.h"
#include <config.h>


gchar * extract_sender (const GValue *changer) {

	gchar *retval = NULL;
	if (changer != NULL) {
		if (G_VALUE_HOLDS_STRING (changer)) {
			retval = g_value_dup_string (changer);
                } else if (G_VALUE_HOLDS_BOXED (changer)) {
                        gpointer boxed = g_value_get_boxed (changer);                           	if (GIRC_IS_CHANNEL_USER (changer)) {
                        retval = g_strdup 
				(GIRC_CHANNEL_USER (boxed)->user->nick);
		} else if (GIRC_IS_USER (changer)) {
			retval = g_strdup (GIRC_USER (boxed)->nick);
                }
	    }
        }
        return retval;

}

gchar * get_target_str (GIrcTarget * target,
                                const gchar * encoding)
{
        if (target == NULL)
                return NULL;
        if (GIRC_IS_CHANNEL (target))
                return girc_convert_to_utf8 (target->channel.name, encoding);
        else
                return girc_convert_to_utf8 (target->query.user->nick, encoding);
}

void other_msg_cb (GIrcClient *irc, GIrcReplyType type, gchar *msg,
		   TinyLemming *lemming) {

	lemming_print (lemming, msg);
	lemming_print (lemming, "\n");
	lemming_scroll_end (lemming);
}

void entry_cb (GtkEntry *entry, TinyLemming *lemming) {

	const gchar *msg = gtk_entry_get_text (entry);
	if (msg == NULL || msg[0] == '\0') return;
	if (msg[0] == '/') {
		lemming_command_handler (lemming, msg);
		gtk_entry_set_text (entry, "");
		return;
	}
	gchar *target_str = get_target_str (lemming->target, "UTF-8");
	gchar *text = g_strdup_printf ("PRIVMSG %s :%s",target_str, msg);
	gchar *local_str = g_strdup_printf ("%s\n", msg);
	lemming_nick_print (lemming, lemming->nick, NICK_OWN_BRACES);
	lemming_print (lemming, local_str);
	girc_client_send_raw(GIRC_CLIENT(lemming->client),text);
	gtk_entry_set_text (entry,"");
	g_free(text);
	g_free(target_str);
}

void user_cb (GIrcClient *client, GIrcChannel *channel, GIrcChannelUser *user,
	      GIrcChannelUserChangedType changed, GValue *changer,
	      GValue *value, TinyLemming *lemming) {
	gchar *old_nick;
	lemming_notice_print (lemming, "--- ");
	switch (changed) {
	case GIRC_CHANNEL_USER_CHANGED_JOIN:
	   lemming_nick_print (lemming, user->user->nick, NICK_NOTICE);
	   lemming_notice_print (lemming, "has joined the channel\n");
	   break;
	case GIRC_CHANNEL_USER_CHANGED_KICKED:
	   lemming_nick_print (lemming, user->user->nick, NICK_NOTICE);
	   lemming_notice_print (lemming, "has been kicked from the channel\n");
	   break;
	case GIRC_CHANNEL_USER_CHANGED_QUIT:
	case GIRC_CHANNEL_USER_CHANGED_PART:
	   lemming_nick_print (lemming, user->user->nick, NICK_NOTICE);
	   lemming_notice_print (lemming, "has left the channel\n");
	   break;
	case GIRC_CHANNEL_USER_CHANGED_NICK:
	   old_nick = g_strdup(g_value_get_string (value));
	   lemming_nick_print (lemming,old_nick,NICK_NOTICE);
	   lemming_notice_print (lemming,"is now known as ");
	   lemming_nick_print (lemming, user->user->nick, NICK_NOTICE);
	   lemming_print (lemming, "\n");
	   g_free(old_nick);
	   break;
	}
}

void target_opened_cb (GIrcClient *client, GIrcTarget *target, 
		     TinyLemming *lemming) {

	if (GIRC_IS_QUERY(target)) return;
	lemming->target = girc_target_ref (target);
	gtk_widget_set_sensitive (GTK_WIDGET(lemming->entry),1);
}

void target_recv_cb (GIrcClient *client, GIrcTarget *target, GValue *sender, 
		     GIrcUserMessageType type, const gchar *msg, 
		     TinyLemming *lemming) {
	
	gchar *output = g_strdup_printf ("%s\n",msg);
	gchar *nick = extract_sender (sender);
	switch (type) {
	case GIRC_USER_MESSAGE_ACTION:
		lemming_nick_print (lemming, nick, NICK_ACTION);
		lemming_action_print (lemming, output);
		break;
	case GIRC_USER_MESSAGE_NOTICE:
		lemming_nick_print (lemming, nick, NICK_NOTICE);
		lemming_notice_print (lemming, output);
		break;
	default:
		if (GIRC_IS_QUERY(target)) {
			lemming_nick_print(lemming, 
					   get_target_str(target,NULL), 
					   NICK_QUERY);
			lemming_print (lemming, output);
			return;
		}
		if (g_strrstr(output,lemming->nick)) {
			lemming_nick_print(lemming, nick, NICK_OWN_BRACES);
		} else {
			lemming_nick_print (lemming, nick, NICK_BRACES);
		}
		lemming_print (lemming, output);
	}
	g_free(output);
	g_free(nick);

}

void ctcp_cb (GIrcClient *irc, GIrcUser *user, GIrcCtcpMessageType type,
	      gchar *msg, TinyLemming *lemming) {

	gchar *output;
	gchar *response;

	switch (type) {
	case GIRC_CTCP_MESSAGE_VERSION:
		output = g_strdup_printf("Received CTCP version from %s\n",
					 user->nick);
		response = g_strdup_printf ("NOTICE %s :TinyLemming " VERSION,
					    user->nick);
		girc_client_send_raw (lemming->client, response);
		g_free (response);
		break;
	default:
		output = g_strdup_printf("Received some sort of CTCP from %s\n",
					 user->nick);
	}

	lemming_print (lemming, output);
	g_free(output);
}

void connect_cb (GIrcClient *irc, GIrcClientStatus status, gchar *server,
		 gchar *nick, TinyLemming *lemming) {

	gchar *hello_msg;
	gchar *join_cmd;

	if (status != GIRC_CLIENT_CONNECTED) return;
	hello_msg = g_strdup_printf ("Trying to join channel %s...\n",
				     lemming->channel);
	join_cmd = g_strdup_printf ("join %s",lemming->channel);

	lemming_print (lemming, hello_msg);
	
        g_signal_connect (G_OBJECT(lemming->client),"ctcp-recv",
                          G_CALLBACK(ctcp_cb), lemming);

	g_signal_connect (G_OBJECT(lemming->client),"msg-recv",
                          G_CALLBACK(other_msg_cb), lemming);

	g_signal_connect (G_OBJECT(lemming->client),"target-recv",
			  G_CALLBACK(target_recv_cb), lemming);

	girc_client_send_raw (GIRC_CLIENT(irc),join_cmd);

	lemming_print (lemming,"\n");
	g_free (hello_msg);
	g_free (join_cmd);
}

void lemming_command_handler (TinyLemming *lemming, const gchar *command) {

	gchar *irc_cmd = NULL;
	gchar *target_str = get_target_str (lemming->target,NULL);
	gchar **cmd_args = g_strsplit (command, " ", 2);

	if (g_strcasecmp(cmd_args[0],"/nick")==0) {
		if (! cmd_args[1]) return;
		if (! girc_client_is_valid_nick(lemming->client,cmd_args[1])) {
			lemming_print (lemming,"--- you can't use that nick\n");
			return;
		}
		irc_cmd = g_strdup_printf("nick %s",cmd_args[1]);
		lemming->nick = g_strdup(cmd_args[1]);
		lemming_notice_print (lemming, "--- you are now known as ");
		lemming_nick_print (lemming, lemming->nick, NICK_NOTICE);
		lemming_print (lemming, "\n");
	}

	if (g_strcasecmp(cmd_args[0],"/action")==0 ||
	    g_strcasecmp(cmd_args[0],"/me")==0) {
		if (! cmd_args[1]) return;
		irc_cmd = g_strdup_printf("PRIVMSG %s :\001ACTION %s\001",
					  target_str, cmd_args[1]);
		lemming_nick_print (lemming, lemming->nick, NICK_ACTION);
		lemming_action_print (lemming, cmd_args[1]);
		lemming_print (lemming,"\n");
	}

	if (g_strcasecmp(cmd_args[0],"/join")==0) {
		gchar *part_cmd = g_strdup_printf("PART %s",target_str);
		if (! cmd_args[1]) return;
		irc_cmd = g_strdup_printf("JOIN %s", cmd_args[1]);
		girc_client_send_raw (lemming->client, part_cmd);
		lemming_print (lemming, "Trying to join ");
		lemming_print (lemming, cmd_args[1]);
		lemming_print (lemming, "\n");
		g_free (part_cmd);
	}

	if (irc_cmd) {
		girc_client_send_raw (lemming->client, irc_cmd);
		g_free (irc_cmd);
	}
	g_strfreev (cmd_args);
}

void lemming_print (TinyLemming *lemming, gchar *msg) {

	GtkTextIter iter;
	gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(lemming->buffer), &iter);
	gtk_text_buffer_insert (GTK_TEXT_BUFFER(lemming->buffer), &iter,
				msg, -1);
	lemming_scroll_end (lemming);
}

void lemming_action_print (TinyLemming *lemming, gchar *msg) {
	GtkTextIter iter;
        gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(lemming->buffer), &iter);
	gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
        	                       &iter, msg, -1, "blue", "italic", NULL);
	lemming_scroll_end (lemming);
}

void lemming_notice_print (TinyLemming *lemming, gchar *msg) {
	GtkTextIter iter;
        gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(lemming->buffer), &iter);
	gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
                                      &iter, msg, -1, "green", "italic", NULL);
	lemming_scroll_end (lemming);
}

void lemming_scroll_end (TinyLemming *lemming) {
		GtkTextIter iter;
		if (lemming->buffer == NULL) return;
		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(lemming->buffer),
                                              &iter);
		if (gtk_text_iter_is_start(&iter)) return;
                gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(lemming->text),
                                              &iter, 0, 0, 0, 0);
}

void lemming_nick_print (TinyLemming *lemming, gchar *nick, NickMode mode) {

	GtkTextIter iter;
	gchar *output;
        gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(lemming->buffer), &iter);

	switch (mode) {
	case NICK_PLAIN:
		gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
					&iter, nick, -1, "bold", NULL);
		lemming_scroll_end (lemming);
		break;
	case NICK_BRACES:
		output = g_strdup_printf("<%s> ",nick);
		gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
              		  	        &iter, output, -1, "bold", NULL);
		lemming_scroll_end (lemming);
		g_free (output);
                break;
	case NICK_ACTION:
		output = g_strdup_printf("* %s ",nick);
                gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
                             &iter, output, -1, "italic", "bold", "blue", NULL);
		lemming_scroll_end (lemming);
                g_free (output);
                break;
	case NICK_NOTICE:
		output = g_strdup_printf("%s ", nick);
                gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
                            &iter, output, -1, "italic", "bold", "green", NULL);
		lemming_scroll_end (lemming);
		g_free(output);
                break;
	case NICK_QUERY:
		output = g_strdup_printf("-%s- ", nick);
		gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
			  &iter, output, -1, "italic", "bold", "magenta", NULL);
		lemming_scroll_end (lemming);
		break;
	case NICK_OWN_PLAIN:
		gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
                                        &iter, nick, -1, "bold", "red", NULL);
                lemming_scroll_end (lemming);
		break;
	case NICK_OWN_ACTION:
		output = g_strdup_printf("* %s ",nick);
                gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
                              &iter, output, -1, "bold", "italic", "red", NULL);
                lemming_scroll_end (lemming);
		g_free (output);
                break;
	case NICK_OWN_BRACES:
                output = g_strdup_printf("<%s> ",nick);
                gtk_text_buffer_insert_with_tags_by_name (lemming->buffer,
                                        &iter, output, -1, "bold", "red", NULL);
                lemming_scroll_end (lemming);
		g_free (output);
                break;
	}
}

TinyLemming * lemming_new (void) {

	TinyLemming *new = g_new0 (TinyLemming,1);
	GtkWidget *vbox = gtk_vbox_new (0,0);
	GtkWidget *scrolled = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolled), 
					GTK_POLICY_NEVER,
					GTK_POLICY_AUTOMATIC);
	new->main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	new->buffer = gtk_text_buffer_new (NULL);
	new->text = gtk_text_view_new ();
	gtk_text_view_set_buffer (GTK_TEXT_VIEW(new->text), new->buffer);
	gtk_text_buffer_create_tag (new->buffer, "bold",
			      	    "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_buffer_create_tag (new->buffer, "red", "foreground", 
				    "red", NULL);
	gtk_text_buffer_create_tag (new->buffer, "italic",
				    "style", PANGO_STYLE_ITALIC, NULL);
	gtk_text_buffer_create_tag (new->buffer, "blue", "foreground",
				    "blue", NULL);
	gtk_text_buffer_create_tag (new->buffer, "green", "foreground",
                                    "green", NULL);
	gtk_text_buffer_create_tag (new->buffer, "magenta", "foreground",
				    "magenta", NULL);
	gtk_text_view_set_editable (GTK_TEXT_VIEW(new->text), FALSE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(new->text),GTK_WRAP_WORD);
	new->entry = gtk_entry_new ();
	gtk_widget_set_sensitive (GTK_WIDGET(new->entry), 0);
	gtk_container_add (GTK_CONTAINER(scrolled),GTK_WIDGET(new->text));
	gtk_box_pack_start (GTK_BOX(vbox),scrolled,1,1,1);
	gtk_box_pack_start (GTK_BOX(vbox),new->entry,0,0,1);
	gtk_container_add (GTK_CONTAINER(new->main),vbox);
	gtk_widget_hide_all (new->main);
	return new;

}

int main (int argc, char *argv[]) {

	TinyLemming *lemming;

	gtk_init (&argc, &argv);
	lemming = lemming_prefs_dialog ();
	lemming->client = GIRC_CLIENT(girc_client_new (lemming->nick,
						       lemming->real_name,
				  		       (gchar*)getenv("USER"),
				  		       lemming->server,
						       (guint)lemming->port, 
							NULL,
		 				       "UTF-8"));
	girc_client_open (GIRC_CLIENT(lemming->client));
	gtk_widget_show_all (lemming->main);
	g_signal_connect (G_OBJECT(lemming->client),"connect-complete",
			  G_CALLBACK(connect_cb), lemming);

	g_signal_connect (G_OBJECT(lemming->client),"target-opened",
			  G_CALLBACK(target_opened_cb), lemming);

	g_signal_connect (G_OBJECT(lemming->client),"channel-user-changed",
			  G_CALLBACK(user_cb), lemming);

	g_signal_connect (G_OBJECT(lemming->main),"delete-event",
			  G_CALLBACK(gtk_main_quit), NULL);

	g_signal_connect (G_OBJECT(lemming->entry),"activate",
			  G_CALLBACK(entry_cb),lemming);

	gtk_main();

}
